@@ -17,14 +17,15 @@
 #include <memory>
 #include <string>
 #include <vector>
-#include <ignition/math/Inertial.hh>
-#include <ignition/math/Pose3.hh>
-#include <ignition/math/Vector3.hh>
+#include <gz/math/Inertial.hh>
+#include <gz/math/Pose3.hh>
+#include <gz/math/Vector3.hh>
 
 #include "sdf/Collision.hh"
 #include "sdf/Error.hh"
 #include "sdf/Light.hh"
 #include "sdf/Link.hh"
+#include "sdf/parser.hh"
 #include "sdf/ParticleEmitter.hh"
 #include "sdf/Sensor.hh"
 #include "sdf/Types.hh"
@@ -42,7 +43,7 @@ class sdf::Link::Implementation
   public: std::string name = "";
 
   /// \brief Pose of the link
-  public: ignition::math::Pose3d pose = ignition::math::Pose3d::Zero;
+  public: gz::math::Pose3d pose = gz::math::Pose3d::Zero;
 
   /// \brief Frame of the pose.
   public: std::string poseRelativeTo = "";
@@ -63,9 +64,9 @@ class sdf::Link::Implementation
   public: std::vector<ParticleEmitter> emitters;
 
   /// \brief The inertial information for this link.
-  public: ignition::math::Inertiald inertial {{1.0,
-            ignition::math::Vector3d::One, ignition::math::Vector3d::Zero},
-            ignition::math::Pose3d::Zero};
+  public: gz::math::Inertiald inertial {{1.0,
+            gz::math::Vector3d::One, gz::math::Vector3d::Zero},
+            gz::math::Pose3d::Zero};
 
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
@@ -73,13 +74,16 @@ class sdf::Link::Implementation
   /// \brief True if this link should be subject to wind, false otherwise.
   public: bool enableWind = false;
 
+  /// \brief True if this link should be subject to gravity, false otherwise.
+  public: bool enableGravity = true;
+
   /// \brief Scoped Pose Relative-To graph at the parent model scope.
   public: sdf::ScopedGraph<sdf::PoseRelativeToGraph> poseRelativeToGraph;
 };
 
 /////////////////////////////////////////////////
 Link::Link()
-  : dataPtr(ignition::utils::MakeImpl<Implementation>())
+  : dataPtr(gz::utils::MakeImpl<Implementation>())
 {
 }
 
@@ -144,9 +148,9 @@ Errors Link::Load(ElementPtr _sdf)
   errors.insert(errors.end(), emitterLoadErrors.begin(),
       emitterLoadErrors.end());
 
-  ignition::math::Vector3d xxyyzz = ignition::math::Vector3d::One;
-  ignition::math::Vector3d xyxzyz = ignition::math::Vector3d::Zero;
-  ignition::math::Pose3d inertiaPose;
+  gz::math::Vector3d xxyyzz = gz::math::Vector3d::One;
+  gz::math::Vector3d xyxzyz = gz::math::Vector3d::Zero;
+  gz::math::Pose3d inertiaPose;
   std::string inertiaFrame = "";
   double mass = 1.0;
 
@@ -174,7 +178,7 @@ Errors Link::Load(ElementPtr _sdf)
     }
   }
   if (!this->dataPtr->inertial.SetMassMatrix(
-      ignition::math::MassMatrix3d(mass, xxyyzz, xyxzyz)))
+      gz::math::MassMatrix3d(mass, xxyyzz, xyxzyz)))
   {
     errors.push_back({ErrorCode::LINK_INERTIA_INVALID,
                      "A link named " +
@@ -188,6 +192,9 @@ Errors Link::Load(ElementPtr _sdf)
   this->dataPtr->enableWind = _sdf->Get<bool>("enable_wind",
       this->dataPtr->enableWind).first;
 
+  this->dataPtr->enableGravity = _sdf->Get<bool>("gravity",
+      this->dataPtr->enableGravity).first;
+
   return errors;
 }
 
@@ -217,6 +224,13 @@ const Visual *Link::VisualByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Visual *Link::VisualByIndex(uint64_t _index)
+{
+  return const_cast<Visual*>(
+      static_cast<const Link*>(this)->VisualByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool Link::VisualNameExists(const std::string &_name) const
 {
@@ -244,6 +258,13 @@ const Collision *Link::CollisionByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Collision *Link::CollisionByIndex(uint64_t _index)
+{
+  return const_cast<Collision*>(
+      static_cast<const Link*>(this)->CollisionByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool Link::CollisionNameExists(const std::string &_name) const
 {
@@ -271,6 +292,13 @@ const Light *Link::LightByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Light *Link::LightByIndex(uint64_t _index)
+{
+  return const_cast<Light*>(
+      static_cast<const Link*>(this)->LightByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool Link::LightNameExists(const std::string &_name) const
 {
@@ -291,6 +319,13 @@ const Sensor *Link::SensorByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Sensor *Link::SensorByIndex(uint64_t _index)
+{
+  return const_cast<Sensor*>(
+      static_cast<const Link*>(this)->SensorByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool Link::SensorNameExists(const std::string &_name) const
 {
@@ -317,6 +352,13 @@ const Sensor *Link::SensorByName(const std::string &_name) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Sensor *Link::SensorByName(const std::string &_name)
+{
+  return const_cast<Sensor*>(
+      static_cast<const Link*>(this)->SensorByName(_name));
+}
+
 /////////////////////////////////////////////////
 uint64_t Link::ParticleEmitterCount() const
 {
@@ -331,6 +373,13 @@ const ParticleEmitter *Link::ParticleEmitterByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+ParticleEmitter *Link::ParticleEmitterByIndex(uint64_t _index)
+{
+  return const_cast<ParticleEmitter*>(
+      static_cast<const Link*>(this)->ParticleEmitterByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool Link::ParticleEmitterNameExists(const std::string &_name) const
 {
@@ -358,22 +407,43 @@ const ParticleEmitter *Link::ParticleEmitterByName(
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+ParticleEmitter *Link::ParticleEmitterByName(const std::string &_name)
+{
+  return const_cast<ParticleEmitter *>(
+      static_cast<const Link*>(this)->ParticleEmitterByName(_name));
+}
 
 /////////////////////////////////////////////////
-const ignition::math::Inertiald &Link::Inertial() const
+const gz::math::Inertiald &Link::Inertial() const
 {
   return this->dataPtr->inertial;
 }
 
 /////////////////////////////////////////////////
-bool Link::SetInertial(const ignition::math::Inertiald &_inertial)
+bool Link::SetInertial(const gz::math::Inertiald &_inertial)
 {
   this->dataPtr->inertial = _inertial;
   return _inertial.MassMatrix().IsValid();
 }
 
 /////////////////////////////////////////////////
-const ignition::math::Pose3d &Link::RawPose() const
+Errors Link::ResolveInertial(
+  gz::math::Inertiald &_inertial,
+  const std::string &_resolveTo) const
+{
+  gz::math::Pose3d linkPose;
+  auto errors = this->SemanticPose().Resolve(linkPose, _resolveTo);
+  if (errors.empty())
+  {
+    _inertial = this->dataPtr->inertial;
+    _inertial.SetPose(linkPose * _inertial.Pose());
+  }
+  return errors;
+}
+
+/////////////////////////////////////////////////
+const gz::math::Pose3d &Link::RawPose() const
 {
   return this->dataPtr->pose;
 }
@@ -385,7 +455,7 @@ const std::string &Link::PoseRelativeTo() const
 }
 
 /////////////////////////////////////////////////
-void Link::SetRawPose(const ignition::math::Pose3d &_pose)
+void Link::SetRawPose(const gz::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
@@ -454,10 +524,17 @@ const Visual *Link::VisualByName(const std::string &_name) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Visual *Link::VisualByName(const std::string &_name)
+{
+  return const_cast<Visual *>(
+      static_cast<const Link*>(this)->VisualByName(_name));
+}
+
 /////////////////////////////////////////////////
 const Collision *Link::CollisionByName(const std::string &_name) const
 {
-  for (auto const &c : this->dataPtr->collisions)
+  for (auto &c : this->dataPtr->collisions)
   {
     if (c.Name() == _name)
     {
@@ -467,6 +544,13 @@ const Collision *Link::CollisionByName(const std::string &_name) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Collision *Link::CollisionByName(const std::string &_name)
+{
+  return const_cast<Collision *>(
+      static_cast<const Link*>(this)->CollisionByName(_name));
+}
+
 /////////////////////////////////////////////////
 const Light *Link::LightByName(const std::string &_name) const
 {
@@ -480,6 +564,14 @@ const Light *Link::LightByName(const std::string &_name) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Light *Link::LightByName(const std::string &_name)
+{
+  return const_cast<Light *>(
+      static_cast<const Link*>(this)->LightByName(_name));
+
+}
+
 /////////////////////////////////////////////////
 sdf::ElementPtr Link::Element() const
 {
@@ -493,7 +585,162 @@ bool Link::EnableWind() const
 }
 
 /////////////////////////////////////////////////
-void Link::SetEnableWind(const bool _enableWind)
+bool Link::EnableGravity() const
+{
+  return this->dataPtr->enableGravity;
+}
+
+/////////////////////////////////////////////////
+void Link::SetEnableWind(bool _enableWind)
 {
   this->dataPtr->enableWind = _enableWind;
 }
+
+/////////////////////////////////////////////////
+void Link::SetEnableGravity(bool _enableGravity)
+{
+  this->dataPtr->enableGravity = _enableGravity;
+}
+
+//////////////////////////////////////////////////
+bool Link::AddCollision(const Collision &_collision)
+{
+  if (this->CollisionNameExists(_collision.Name()))
+    return false;
+  this->dataPtr->collisions.push_back(_collision);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Link::AddVisual(const Visual &_visual)
+{
+  if (this->VisualNameExists(_visual.Name()))
+    return false;
+  this->dataPtr->visuals.push_back(_visual);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Link::AddLight(const Light &_light)
+{
+  if (this->LightNameExists(_light.Name()))
+    return false;
+  this->dataPtr->lights.push_back(_light);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Link::AddSensor(const Sensor &_sensor)
+{
+  if (this->SensorNameExists(_sensor.Name()))
+    return false;
+  this->dataPtr->sensors.push_back(_sensor);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Link::AddParticleEmitter(const ParticleEmitter &_emitter)
+{
+  if (this->ParticleEmitterNameExists(_emitter.Name()))
+    return false;
+  this->dataPtr->emitters.push_back(_emitter);
+  return true;
+}
+
+//////////////////////////////////////////////////
+void Link::ClearCollisions()
+{
+  this->dataPtr->collisions.clear();
+}
+
+//////////////////////////////////////////////////
+void Link::ClearVisuals()
+{
+  this->dataPtr->visuals.clear();
+}
+
+//////////////////////////////////////////////////
+void Link::ClearLights()
+{
+  this->dataPtr->lights.clear();
+}
+
+//////////////////////////////////////////////////
+void Link::ClearSensors()
+{
+  this->dataPtr->sensors.clear();
+}
+
+//////////////////////////////////////////////////
+void Link::ClearParticleEmitters()
+{
+  this->dataPtr->emitters.clear();
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Link::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("link.sdf", elem);
+
+  elem->GetAttribute("name")->Set(this->Name());
+
+  // Set pose
+  sdf::ElementPtr poseElem = elem->GetElement("pose");
+  if (!this->dataPtr->poseRelativeTo.empty())
+  {
+    poseElem->GetAttribute("relative_to")->Set<std::string>(
+        this->dataPtr->poseRelativeTo);
+  }
+  poseElem->Set<gz::math::Pose3d>(this->RawPose());
+
+  // inertial
+  sdf::ElementPtr inertialElem = elem->GetElement("inertial");
+  inertialElem->GetElement("pose")->Set(
+      this->dataPtr->inertial.Pose());
+  const gz::math::MassMatrix3d &massMatrix =
+    this->dataPtr->inertial.MassMatrix();
+  inertialElem->GetElement("mass")->Set<double>(massMatrix.Mass());
+  sdf::ElementPtr inertiaElem = inertialElem->GetElement("inertia");
+  inertiaElem->GetElement("ixx")->Set(massMatrix.Ixx());
+  inertiaElem->GetElement("ixy")->Set(massMatrix.Ixy());
+  inertiaElem->GetElement("ixz")->Set(massMatrix.Ixz());
+  inertiaElem->GetElement("iyy")->Set(massMatrix.Iyy());
+  inertiaElem->GetElement("iyz")->Set(massMatrix.Iyz());
+  inertiaElem->GetElement("izz")->Set(massMatrix.Izz());
+
+  // wind mode
+  elem->GetElement("enable_wind")->Set(this->EnableWind());
+
+  // Collisions
+  for (const sdf::Collision &collision : this->dataPtr->collisions)
+  {
+    elem->InsertElement(collision.ToElement(), true);
+  }
+
+  // Light
+  for (const sdf::Light &light : this->dataPtr->lights)
+  {
+    elem->InsertElement(light.ToElement(), true);
+  }
+
+  // Particle emitters
+  for (const sdf::ParticleEmitter &emitter : this->dataPtr->emitters)
+  {
+    elem->InsertElement(emitter.ToElement(), true);
+  }
+
+  // Sensors
+  for (const sdf::Sensor &sensor : this->dataPtr->sensors)
+  {
+    elem->InsertElement(sensor.ToElement(), true);
+  }
+
+  // Visuals
+  for (const sdf::Visual &visual : this->dataPtr->visuals)
+  {
+    elem->InsertElement(visual.ToElement(), true);
+  }
+
+  return elem;
+}
