@@ -182,6 +182,18 @@ SDF::SDF()
 {
 }
 
+/////////////////////////////////////////////////
+SDF::SDF(const SDF& other)
+{
+  this->dataPtr = std::make_unique<SDFPrivate>(*other.dataPtr);
+}
+
+/////////////////////////////////////////////////
+SDF& SDF::operator=(const SDF& other)
+{
+  this->dataPtr = std::make_unique<SDFPrivate>(*other.dataPtr);
+  return *this;
+}
 /////////////////////////////////////////////////
 SDF::~SDF()
 {
@@ -194,9 +206,9 @@ void SDF::PrintDescription()
 }
 
 /////////////////////////////////////////////////
-void SDF::PrintValues()
+void SDF::PrintValues(const PrintConfig &_config)
 {
-  this->Root()->PrintValues("");
+  this->Root()->PrintValues("", _config);
 }
 
 /////////////////////////////////////////////////
@@ -328,7 +340,7 @@ void SDF::Write(const std::string &_filename)
 }
 
 /////////////////////////////////////////////////
-std::string SDF::ToString() const
+std::string SDF::ToString(const PrintConfig &_config) const
 {
   std::ostringstream stream;
 
@@ -338,7 +350,7 @@ std::string SDF::ToString() const
     stream << "<sdf version='" << SDF::Version() << "'>\n";
   }
 
-  stream << this->Root()->ToString("");
+  stream << this->Root()->ToString("", _config);
 
   if (this->Root()->GetName() != "sdf")
   {
