@@ -31,13 +31,9 @@ TEST(SDFParser, UrdfGazeboExtensionURDFTest)
 
   sdf::Root root;
   auto errors = root.Load(urdfTestFile);
-  EXPECT_TRUE(errors.empty());
-  for (auto e : errors)
-  {
-    std::cerr << e << std::endl;
-  }
+  EXPECT_TRUE(errors.empty()) << errors;
 
-  auto modelDom = root.ModelByIndex(0);
+  auto modelDom = root.Model();
   ASSERT_NE(nullptr, modelDom);
 
   // Verify that model is not static
@@ -115,11 +111,16 @@ TEST(SDFParser, UrdfGazeboExtensionURDFTest)
     }
   }
 
+  sdf::ElementPtr link0;
   for (sdf::ElementPtr link = model->GetElement("link"); link;
        link = link->GetNextElement("link"))
   {
-    std::string linkName = link->Get<std::string>("name");
-    if (linkName == "link1" || linkName == "link2")
+    const auto& linkName = link->Get<std::string>("name");
+    if (linkName == "link0")
+    {
+      link0 = link;
+    }
+    else if (linkName == "link1" || linkName == "link2")
     {
       EXPECT_TRUE(link->HasElement("enable_wind"));
 
@@ -150,18 +151,6 @@ TEST(SDFParser, UrdfGazeboExtensionURDFTest)
       EXPECT_FALSE(link->HasElement("velocity_decay"));
     }
   }
-
-  sdf::ElementPtr link0;
-  for (sdf::ElementPtr link = model->GetElement("link"); link;
-       link = link->GetNextElement("link"))
-  {
-    const auto& linkName = link->Get<std::string>("name");
-    if (linkName == "link0")
-    {
-      link0 = link;
-      break;
-    }
-  }
   ASSERT_TRUE(link0);
 
   auto checkElementPoses = [&](const std::string &_elementName) -> void
@@ -345,13 +334,9 @@ TEST(SDFParser, FixedJointExample)
 
   sdf::Root root;
   auto errors = root.Load(urdfTestFile);
-  EXPECT_TRUE(errors.empty());
-  for (auto e : errors)
-  {
-    std::cerr << e << std::endl;
-  }
+  EXPECT_TRUE(errors.empty()) << errors;
 
-  auto model = root.ModelByIndex(0);
+  auto model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("fixed_joint_example", model->Name());
 
@@ -393,13 +378,9 @@ TEST(SDFParser, FixedJointSimple)
 
   sdf::Root root;
   auto errors = root.Load(urdfTestFile);
-  EXPECT_TRUE(errors.empty());
-  for (auto e : errors)
-  {
-    std::cerr << e << std::endl;
-  }
+  EXPECT_TRUE(errors.empty()) << errors;
 
-  auto model = root.ModelByIndex(0);
+  auto model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("fixed_joint_simple", model->Name());
   EXPECT_FALSE(model->Static());
@@ -428,13 +409,9 @@ TEST(SDFParser, FixedJointStatic)
 
   sdf::Root root;
   auto errors = root.Load(urdfTestFile);
-  EXPECT_TRUE(errors.empty());
-  for (auto e : errors)
-  {
-    std::cerr << e << std::endl;
-  }
+  EXPECT_TRUE(errors.empty()) << errors;
 
-  auto model = root.ModelByIndex(0);
+  auto model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("fixed_joint_simple", model->Name());
 
